#include "definitions.h"
#include "macros.h"
#include "interupts.h"

.extern initMMU
.extern loadProgram
.extern schedule
.extern initProcessManager
.extern initSchedulerTimer
.global kernelSize

.text

// The Makefile will add the size of the kernel in here so we can determine where the kernel ends.
imagedata:
    b       entryPoint
kernelSize: .word 0

entryPoint:
    // copy IDT to begining of ram
    ldr     r0,=IDT_START
    ldr     r1,=IDT_END
    mov     r2,#0
1:  ldr     r3,[r0],#4
    str     r3,[r2],#4
    cmp     r0,r1
    bne     1b


    // initialize stack for all modes, except user mode because we will set it on process creation
    msr     CPSR_c,#0b11010001           // stack for FIQ mode
    ldr     r13,=STACK_BASE_FIQ
    msr     CPSR_c,#0b11010111           // stack for Abort mode
    ldr     r13,=STACK_BASE_ABORT  
    msr     CPSR_c,#0b11011011           // stack for Undefined mode
    ldr     r13,=STACK_BASE_UNDEFINED 
    msr     CPSR_c,#0b11010011           // stack for Supervisor mode. And we will stay in that mode
    ldr     r13,=STACK_BASE_UNDEFINED   // This is temporary, We will use another stack after MMU is init

    // Disable TEX remap
    mrc     p15,0,r0,c1,c0,0
    bic     r0,#28
    mcr     p15,0,r0,c1,c0,0


    PRINTK("Booting\r\n")
    PRINTK("Init MMUr\n")
    bl      initMMU
    ldr     r13,=SVC_STACK_TOP          // now that mmu is setup, use proper stack

    PRINTK("Init GIC\r\n")
    bl      initGIC

    PRINTK("Init Process Manager\r\n")
    bl      initProcessManager


    PRINTK("Init Scheduler\r\n")
    ldr     r1,=TIMER0_RELOAD_VALUE
    bl      initSchedulerTimer

    
    PRINTK("Loading programs\r\n")
    mov     r4,#1
    bl      loadProgram
    mov     r11,r0

    mov     r4,#2
    bl      loadProgram
    mov     r10,r0

    PRINTK("System is up. Entering spin-wait\r\n")

    // Enable interrupts
    mrs     r0,CPSR
    bic     r0,#0b11000000
    msr     CPSR,r0

kernelIdleLoop: 
    ldr     r1,=0x10011004
    ldr     r2,[r1]

    ldr     r1,=0x1E001284
    ldr     r3,[r1]
    ldr     r1,=0x1E001304
    ldr     r4,[r1]

    b       kernelIdleLoop


// The IDT will be copied to the beginning of the RAM once the kernel is launched
IDT_START:
IDT_RESET
IDT_UNDEFINED_INSTRUCTION
IDT_SWI
IDT_INSTRUCTION_FETCH_ABORT
IDT_DATA_ABORT
IDT_RESERVED
IDT_IRQ
IDT_FIRQ
.ltorg // this will dump the literal pool here. Needed for IDT
IDT_END:
