#include "macros.h"
#include "board.h"
#include "definitions.h"

.global int_undefined
.global int_IRQ
.global initGIC
.global enableInterrupt
.global returnFromInterrupt
.global generate_SGI

#define GIC_CPUCONTROL 0x000
#define GIC_PRIORITYMASK 0x004
#define GIC_BINARYPOINT 0x008
#define GIC_INTACK 0x00C
#define GIC_EOI     0x10
#define DISTRIBUTOR_CONTROL 0x00
#define SET_ENABLE1 0x104//    bit0 of SET_ENABLE1 represents int32
#define SET_ENABLE2 0x108//    bit0 of SET_ENABLE2 represents int64
#define GID_ICDSGIR 0xF00

initGIC:
    push(r0-r3,r14)

    // clear the handler table
    ldr     r0,=handlers
    mov     r1,#0
    mov     r2,#0
1:  str     r1,[r0],#4
    add     r2,#1
    ands    r2,#0xFF
    bne     1b

    mov     r1,#1
    ldr     r0,=GIC0
    str     r1,[r0,#GIC_CPUCONTROL]      // enable GIC0

    mov     r1,#1
    ldr     r0,=GIC0+GIC_DISTRIBUTOR
    str     r1,[r0,#DISTRIBUTOR_CONTROL]      // enable Distributor0

    ldr     r0,=GIC0
    mov     r1,#0xFF
    str     r1,[r0,#GIC_PRIORITYMASK]      // unmask al linterrupts

    pop(r0-r3,r15)



// r0 = interrupt number
generate_SGI:
    push(r0-r2,r14)
    
    ldr     r2,=GIC0+GIC_DISTRIBUTOR+GID_ICDSGIR
    mov     r1,#1
    orr     r0,r0,r1,LSL #25        // send int to calling CPU
    str     r0,[r2]

    pop(r0-r2,r15)


// r2 = int to enable
// r3 = handler address
enableInterrupt:
    push(r0-r2,r14)
    ldr     r0,=handlers
    str     r3,[r0,r2,LSL #2]
    ldr     r0,=GIC0+GIC_DISTRIBUTOR+SET_ENABLE1
    cmp     r2,#64
    sublo   r2,#32
    subhs   r2,#64
    addhs   r0,#4
    mov     r1,#1
    lsl     r1,r2
    ldr     r2,[r0]
    orr     r2,r1
    str     r2,[r0]
    pop(r0-r2,r15)

handlers:
.skip (256*4)



int_undefined: 
    PRINTK("Undefined interrupt\r\n")
1: b 1b



// The IRQ handler finds the registered handler and transfers control to it
// and guarantees that r0-r12 are untouched. r14 DOES NOT contain a return address.
// the registered handle must jump to returnFromInterrupt
int_IRQ:
    ldr     r13,=IRQ_STACK_TOP          // no re-entrant ints, so reset the stack everytime. This allows us 
    sub     r14,#4                      // must remove #4 from return address according to documentation
    push(r14)                           //    use the current task's IRQ stack
    push(r1)
    // find handler
    ldr     r14,=GIC0
    ldr     r1,[r14,#GIC_INTACK]
    ubfx    r1,r1,#0,#10
    ldr     r14,=currentInterrupt
    str     r1,[r14]                    // save current int for when we return from int
    ldr     r14,=handlers
    ldr     r14,[r14,r1,LSL #2]
    pop(r1)                  // we need to guarantee that r0-r12 are clean before going in handler (because of scheduler)
    cmp     r14,#0  
    movne   r15,r14           // jump to handler. That handler will need to call returnFromInterrupt
    pop(r15)^                   //TODO:  if handler was 0, return now, but must EOI

returnFromInterrupt:                    // from this point, integrity of r0-r12 must be guaranteed!!!
//    PRINTK("IRQ\r\n")

    push(r0-r2)
    ldr     r0,=currentInterrupt
    ldr     r1,[r0]
    cmp     r1,#0               // if currentInterrupt is null, dont write it in register
    movne   r2,#0
    strne   r2,[r0]
    ldrne   r0,=GIC0
    strne   r1,[r0,#GIC_EOI]        // EOI
    pop(r0-r2)
    pop(r15)^

currentInterrupt: .word 0 // we don't allow re-entrant IRQs so this is safe

